@@ -17,17 +17,17 @@
 #define R2_LIBDIR "/usr/lib"
 #define R2_INCDIR "/usr/include/libr"
 #define R2_DATDIR "/usr/share"
-#define R2_WWWROOT "/usr/share/radare2/4.3.1/www"
-#define R2_PLUGINS "lib/radare2/4.3.1"
-#define R2_EXTRAS "lib/radare2-extras/4.3.1"
-#define R2_BINDINGS "lib/radare2-bindings/4.3.1"
+#define R2_WWWROOT "/usr/share/radare2/5.0.0/www"
+#define R2_PLUGINS "lib/radare2/5.0.0"
+#define R2_EXTRAS "lib/radare2-extras/5.0.0"
+#define R2_BINDINGS "lib/radare2-bindings/5.0.0"
 #define R2_DATDIR_R2 "share/radare2"
-#define R2_SDB "share/radare2/4.3.1"
-#define R2_ZIGNS "share/radare2/4.3.1/zigns"
-#define R2_THEMES "share/radare2/4.3.1/cons"
+#define R2_SDB "share/radare2/5.0.0"
+#define R2_ZIGNS "share/radare2/5.0.0/zigns"
+#define R2_THEMES "share/radare2/5.0.0/cons"
 #define R2_FORTUNES "share/doc/radare2"
-#define R2_FLAGS "share/radare2/4.3.1/flag"
-#define R2_HUD "share/radare2/4.3.1/hud"
+#define R2_FLAGS "share/radare2/5.0.0/flag"
+#define R2_HUD "share/radare2/5.0.0/hud"
 #else
 #if __WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
@@ -72,6 +72,7 @@
 #define R2_HOME_PDB         R_JOIN_2_PATHS (R2_HOME_DATADIR, "pdb")
 #define R2_HOME_PROJECTS    R_JOIN_2_PATHS (R2_HOME_DATADIR, "projects")
 #define R2_HOME_WWWROOT     R_JOIN_2_PATHS (R2_HOME_DATADIR, "www")
+#define R2_HOME_SDB_FCNSIGN R_JOIN_2_PATHS (R2_HOME_DATADIR, "fcnsign")
 #define R2_HOME_SDB_FORMAT  R_JOIN_2_PATHS (R2_HOME_DATADIR, "format")
 #define R2_HOME_HUD         R_JOIN_2_PATHS (R2_HOME_DATADIR, "hud")
 #define R2_HOME_BINRC       R_JOIN_2_PATHS (R2_HOME_DATADIR, "rc.d")
@@ -97,9 +98,7 @@
 #define HAVE_PTRACE 1
 #define USE_PTRACE_WRAP 1
 #define HAVE_FORK 1
-
-#define USE_TREESITTER 0
-
+#define WANT_DYLINK 1
 #define WITH_GPL 1
 
 #if __APPLE__ && __POWERPC__
